#include <windows.h>
#include <string.h>
#include <stdlib.h>
#include "exports.h"
#include "cwstart.h"

LIBMAIN

#define NUM_HASH_ENTRIES 29
#define HASH_ENTRY_SIZE 30

/* Exported functions */
void DLL CommentLine(void);
void DLL CommentLineWith(LPSTR startComment, LPSTR endComment);
void DLL UnCommentLine(void);
void DLL UnCommentLineWith(LPSTR startComment, LPSTR endComment);
void DLL AddCommentDelimeters(LPSTR ext, LPSTR start1, LPSTR end1, LPSTR start2, LPSTR end2);
BOOL DLL GetCommentDelimeters(LPSTR ext, LPSTR start1, LPSTR end1, LPSTR start2, LPSTR end2);

/* Private helper functions */
void DLL LowerStr(char* str);
void DLL RemovePeriod(LPSTR str);
LPVOID DLL CreateDHashTable(void);
void DLL MakeEntry(LPSTR dest, LPSTR str1, LPSTR str2, LPSTR str3, LPSTR str4);
void DLL ParseEntry(LPSTR entry, LPSTR str1, LPSTR str2, LPSTR str3, LPSTR str4);

/* Private variables */
static LPVOID dHashTable = NULL;

void DLL 
_init()
{
	static BOOL firstTime = TRUE;

	if (firstTime) {
		LibExport("CommentLine");
		LibExport("CommentLineWith LPSTR LPSTR");
		LibExport("UnCommentLine");
		LibExport("UnCommentLineWith LPSTR LPSTR");
		LibExport("AddCommentDelimeters LPSTR LPSTR LPSTR LPSTR LPSTR");
		LibExport("BOOL GetCommentDelimeters LPSTR LPSTR LPSTR LPSTR LPSTR");

		dHashTable = CreateDHashTable();
		AddCommentDelimeters("c", "/*", "*/", "/\\*", "\\*/");
		AddCommentDelimeters("cpp", "//", "", "//", "");
		AddCommentDelimeters("pas", "{", "}", "{", "}");
		AddCommentDelimeters("bas", "REM ", "", "REM\\ ", "");
		AddCommentDelimeters("asm", ";", "", ";", "");
		AddCommentDelimeters("for", "c ", "", "c\\ ", "");
		AddCommentDelimeters("cob", "*", "", "\\*", "");
		AddCommentDelimeters("ada", "//", "", "//", "");
		AddCommentDelimeters("java", "//", "", "//", "");
		AddCommentDelimeters("prg", "//", "", "//", "");
		AddCommentDelimeters("sc", "//", "", "//", "");

		firstTime = FALSE;
	}
}

void DLL
CommentLine(void)
{
	char start1[255];
	char end1[255];
	char start2[255];
	char end2[255];
	char* tmp;
	char* tmp2;

	tmp = TransformFilename(BufQFilename(), "%e");
	LowerStr(tmp);
	tmp2 = ExtQAlias(tmp);
	if (tmp2)
		tmp = tmp2;

	if (tmp)
		if (GetCommentDelimeters(tmp, start1, end1, start2, end2))
			CommentLineWith(start1, end1);
}

void DLL CommentLineWith(LPSTR startComment, LPSTR endComment)
{
	long currCol = BufQCurrentCol() - 1;

	if (!startComment)
		return;
	MovHome();
	BufInsertStr(startComment);
	if (endComment) {
		MovEOL();
		BufInsertStr(endComment);
	}
	MovHome();
	MovRight(currCol);
}

void DLL UnCommentLine(void)
{
	char start1[255];
	char end1[255];
	char start2[255];
	char end2[255];
	char* tmp;
	char* tmp2;

	tmp = TransformFilename(BufQFilename(), "%e");
	LowerStr(tmp);
	tmp2 = ExtQAlias(tmp);
	if (tmp2)
		tmp = tmp2;

	if (tmp)
		if (GetCommentDelimeters(tmp, start1, end1, start2, end2))
			UnCommentLineWith(start2, end2);
}

void DLL UnCommentLineWith(LPSTR startComment, LPSTR endComment)
{
	long currCol = BufQCurrentCol() - 1;
	int startCol = 0;
	int endCol = 0;
	long foundLen;
	char tmpStr[256];
	DWORD oldFlags = SrchQFlags();
	int autoIndent = BufQAutoIndent();

	if (!startComment)
		return;

	SrchSetFwd(TRUE);
	SrchSetCase(TRUE);
   	SrchSetWrap(FALSE);     
	SrchSetRegex(TRUE);
	SrchSetSelection(TRUE);

	MarkBeginSel(SELECTION_LINE);

	MovHome();
	strcpy(tmpStr, startComment);
	if (SrchFind(tmpStr, SrchQFlags(), &foundLen)) {
		BufDelChar(foundLen);
	}

	if (endComment) {
		MovHome();
	 	strcpy(tmpStr, "[\\ \\t]*");
		strcat(tmpStr, endComment);
		strcat(tmpStr, "[\\ \\t]*$");
		if (SrchFind(tmpStr, SrchQFlags(), &foundLen)) {
			BufDelChar(foundLen);
		}
	}

	MovHome();
	MovRight(currCol);
	MarkRemoveSel();

	SrchSetFlags(oldFlags);
}

void DLL LowerStr(char* str)
{
	int i = 0;
	while (str[i] != '\0') {
		str[i] = tolower(str[i]);
		i++;
	}
}

LPVOID DLL CreateDHashTable(void)
{
	return(HashCreateTable(NUM_HASH_ENTRIES, HASH_ENTRY_SIZE, HASH_NO_VAL));
}

void DLL AddCommentDelimeters(LPSTR ext, LPSTR start1, LPSTR end1, LPSTR start2, LPSTR end2)
{
	LPSTR entry;
	char extStr[15];

	strcpy(extStr, ext);
	LowerStr(extStr);
	RemovePeriod(extStr);
	entry = HashGetEntry(dHashTable, extStr, strlen(extStr));
	MakeEntry(entry, start1, end1, start2, end2);
}

BOOL DLL GetCommentDelimeters(LPSTR ext, LPSTR start1, LPSTR end1, LPSTR start2, LPSTR end2)
{
	LPSTR entry;
	char extStr[15];

	strcpy(extStr, ext);
	LowerStr(extStr);
	RemovePeriod(extStr);
	entry = HashFindEntry(dHashTable, extStr, strlen(extStr));
	if (entry) {
		ParseEntry(entry, start1, end1, start2, end2);
		return(TRUE);
	}
	return(FALSE);
}

void DLL MakeEntry(LPSTR dest, LPSTR str1, LPSTR str2, LPSTR str3, LPSTR str4)
{
	char tmp[2];
	tmp[0] = (char)0x7E;
	tmp[1] = '\0';
	strcpy(dest, str1);
	strcat(dest, tmp);
	strcat(dest, str2);
	strcat(dest, tmp);
	strcat(dest, str3);
	strcat(dest, tmp);
	strcat(dest, str4);
	strcat(dest, tmp);
}

void DLL ParseEntry(LPSTR entry, LPSTR str1, LPSTR str2, LPSTR str3, LPSTR str4)
{
	int i = 0;
	int currStr;
	LPSTR strings[4];
	char tmp[2];
	tmp[1] = '\0';

	strings[0] = str1;
	strings[1] = str2;
	strings[2] = str3;
	strings[3] = str4;
	for (currStr = 0; currStr < 4; currStr++) {
		strcpy(strings[currStr], "");
		while (entry[i] != (char)0x7E) {
			tmp[0] = entry[i];
			strcat(strings[currStr], tmp);
			i++;
		}
		i++;
	}
}

void DLL RemovePeriod(LPSTR str)
{
	char tempStr[256];
	char* tempStrPtr = tempStr;

	strcpy(tempStrPtr, str);
	if (tempStrPtr[0] == '.') {
		tempStrPtr++;
		strcpy(str, tempStrPtr);
	}
}